@@ -1,6 +1,7 @@
 from __future__ import division, print_function
 
 import numpy as np
+from numpy.linalg import LinAlgError
 from scipy.optimize._numdiff import approx_derivative
 
 from refnx.util import ErrorProp as EP
@@ -13,7 +14,9 @@ from refnx.analysis import (is_parameter, Parameter, possibly_create_parameter,
 
 class BaseObjective(object):
     """Don't necessarily have to use Parameters, could use np.array"""
-    def __init__(self, p, lnlike, lnprior=None, fcn_args=(), fcn_kwds=None):
+    def __init__(self, p, lnlike, lnprior=None, fcn_args=(), fcn_kwds=None,
+                 name=None):
+        self.name = name
         self.parameters = p
         self.nvary = len(p)
         self._lnlike = lnlike
@@ -31,6 +34,7 @@ class BaseObjective(object):
         ----------
         pvals : np.ndarray
             Array containing the values to be tested.
+
         """
         self.parameters[:] = pvals
 
@@ -47,6 +51,7 @@ class BaseObjective(object):
         -------
         nll : float
             negative log-likelihood
+
         """
         vals = self.parameters
         if pvals is not None:
@@ -67,6 +72,7 @@ class BaseObjective(object):
         -------
         lnprior : float
             log-prior probability
+
         """
         vals = self.parameters
         if pvals is not None:
@@ -89,6 +95,7 @@ class BaseObjective(object):
         -------
         lnlike : float
             log-likelihood probability.
+
         """
         vals = self.parameters
         if pvals is not None:
@@ -114,6 +121,7 @@ class BaseObjective(object):
         -----
         The log probability is the sum is the sum of the log-prior and
         log-likelihood probabilities. Does not set the parameter attribute.
+
         """
         vals = self.parameters
         if pvals is not None:
@@ -131,6 +139,7 @@ class BaseObjective(object):
         -------
         varying_parameters : np.ndarray
             The parameters varying in this objective function.
+
         """
         return self.parameters
 
@@ -140,6 +149,7 @@ class BaseObjective(object):
         -------
         covar : np.ndarray
             The covariance matrix for the fitting system
+
         """
         _pvals = np.array(self.varying_parameters())
         hess = approx_hess2(_pvals, self.nll)
@@ -151,34 +161,51 @@ class BaseObjective(object):
 class Objective(BaseObjective):
     """
     Objective function for using with curvefitters such as
-    `refnx.analysis.curvefitter.CurveFitter`
+    `refnx.analysis.curvefitter.CurveFitter`.
+
+    Parameters
+    ----------
+    model : refnx.analysis.Model
+        the generative model function. One can also provide an object that
+        inherits `refnx.analysis.Model`.
+    data : refnx.dataset.Data1D
+        data to be analysed.
+    lnsigma : float or refnx.analysis.Parameter, optional
+        Used if the  experimental uncertainty (`data.y_err`) underestimated by
+        a constant fractional amount. The experimental uncertainty is modified
+        as:
+
+        `s_n**2 = y_err**2 + exp(lnsigma * 2) * model**2`
+
+        See `Objective.lnlike` for more details.
+    use_weights : bool
+        use experimental uncertainty in calculation of residuals and
+        lnlike, if available. If this is set to False, then you should also
+        set `self.lnsigma.vary = False`, it will have no effect on the fit.
+    transform : callable, optional
+        the model, data and data uncertainty are transformed by this
+        function before calculating the likelihood/residuals. Has the
+        signature `transform(data.x, y, y_err=None)`, returning the tuple
+        (`transformed_y, transformed_y_err`).
+    lnprob_extra : callable, optional
+        user specifiable log-probability term. This contribution is in
+        addition to the log-prior term of the `model` parameters, and
+        `model.lnprob`, as well as the log-likelihood of the `data`. Has
+        signature:
+        `lnprob_extra(model, data)`. The `model` will already possess
+        updated parameters. Beware of including the same log-probability
+        terms more than once.
+    name : str
+        Name for the objective.
+
+    Notes
+    -----
+    For parallelisation `lnprob_extra` needs to be picklable.
+
     """
 
-    def __init__(self, model, data, lnsigma=0, use_weights=True,
-                 transform=None):
-        """
-        Parameters
-        ----------
-        model : refnx.analysis.Model
-            the generative model function. One can also provide an object that
-            inherits `refnx.analysis.Model`.
-        data : refnx.dataset.Data1D
-            data to be analysed.
-        lnsigma : float or Parameter, optional
-            the experimental uncertainty (`data.y_err`) is multiplied by
-            `exp(lnsigma)`. Used when the experimental uncertainty is
-            underestimated.
-        use_weights : bool
-            use experimental uncertainty in calculation of residuals and
-            lnlike, if available. If this is set to False, then you should also
-            set `self.lnsigma.vary = False`, it will have no effect on the fit.
-        transform : callable, optional
-            the model, data and data uncertainty are transformed by this
-            function before calculating the likelihood/residuals. Has the
-            signature `transform(data.x, y, y_err=None)`, returning the tuple
-            `transformed_y, transformed_y_err`.
-        """
-        # lnsigma is a parameter for underestimated errors
+    def __init__(self, model, data, lnsigma=None, use_weights=True,
+                 transform=None, lnprob_extra=None, name=None):
         self.model = model
         # should be a Data1D instance
         if isinstance(data, Data1D):
@@ -186,17 +213,20 @@ class Objective(BaseObjective):
         else:
             self.data = Data1D(data=data)
 
-        if is_parameter(lnsigma):
-            self.lnsigma = lnsigma
-        else:
-            self.lnsigma = lnsigma
+        self.lnsigma = lnsigma
+        if lnsigma is not None:
+            self.lnsigma = possibly_create_parameter(lnsigma, 'lnsigma')
 
         self.use_weights = use_weights
         self.transform = transform
+        self.lnprob_extra = lnprob_extra
+        self.name = name
+        if name is None:
+            self.name = id(self)
 
     def __repr__(self):
         s = ["{:_>80}".format('')]
-        s.append('Objective - {0}'.format(id(self)))
+        s.append('Objective - {0}'.format(self.name))
 
         # dataset name
         if self.data.name is None:
@@ -215,20 +245,15 @@ class Objective(BaseObjective):
     @property
     def weighted(self):
         """
-        Returns
-        -------
-        weighted : bool
-            Does the data have weights (`data.y_err`)?
+        **bool** does the data have weights (`data.y_err`)?
         """
         return self.data.weighted
 
     @property
     def npoints(self):
         """
-        Returns
-        -------
-        npoints : int
-            The number of points in the dataset.
+        **int** the number of points in the dataset.
+
         """
         return self.data.y.size
 
@@ -236,9 +261,9 @@ class Objective(BaseObjective):
         """
         Returns
         -------
-        varying_parameters : Parameters
-            A Parameters instance containing the varying Parameter objects
-            that are allowed to vary during the fit.
+        varying_parameters : refnx.analysis.Parameters
+            The varying Parameter objects allowed to vary during the fit.
+
         """
         # create and return a Parameters object because it has the
         # __array__ method, which allows one to quickly get numerical values.
@@ -252,10 +277,9 @@ class Objective(BaseObjective):
         y = self.data.y
         weight = self.use_weights and self.data.weighted
 
+        y_err = 1.
         if weight:
-            y_err = self.data.y_err * np.exp(float(self.lnsigma))
-        else:
-            y_err = 1.
+            y_err = self.data.y_err
 
         if self.transform is None:
             return y, y_err, model
@@ -271,12 +295,13 @@ class Objective(BaseObjective):
 
     def generative(self, pvals=None):
         """
-        Calculate the generative function associated with the model
+        Calculate the generative (dependent variable) function associated with
+        the model.
 
         Parameters
         ----------
-        pvals : np.ndarray
-            Parameter values for evaluation
+        pvals : array-like or refnx.analysis.Parameters
+            values for the varying or entire set of parameters
 
         Returns
         -------
@@ -291,13 +316,14 @@ class Objective(BaseObjective):
 
         Parameters
         ----------
-        pvals : np.ndarray
-            Parameter values for evaluation
+        pvals : array-like or refnx.analysis.Parameters
+            values for the varying or entire set of parameters
 
         Returns
         -------
         residuals : np.ndarray
             Residuals, `(data.y - model) / y_err`.
+
         """
         self.setp(pvals)
 
@@ -306,7 +332,13 @@ class Objective(BaseObjective):
 
         y, y_err, model = self._data_transform(model)
 
-        return (y - model) / y_err
+        if self.lnsigma is not None:
+            s_n = np.sqrt(y_err * y_err +
+                          np.exp(2 * float(self.lnsigma)) * model * model)
+        else:
+            s_n = y_err
+
+        return np.squeeze((y - model) / s_n)
 
     def chisqr(self, pvals=None):
         """
@@ -314,13 +346,14 @@ class Objective(BaseObjective):
 
         Parameters
         ----------
-        pvals : np.ndarray
-            Parameter values for evaluation
+        pvals : array-like or refnx.analysis.Parameters
+            values for the varying or entire set of parameters
 
         Returns
         -------
         chisqr : np.ndarray
             Chi-squared value, `np.sum(residuals**2)`.
+
         """
         # TODO reduced chisqr? include z-scores for parameters? DOF?
         self.setp(pvals)
@@ -330,12 +363,9 @@ class Objective(BaseObjective):
     @property
     def parameters(self):
         """
-        All the Parameters contained in the fitting system.
+        :class:`refnx.analysis.Parameters`, all the Parameters contained in the
+        fitting system.
 
-        Returns
-        -------
-        parameters : Parameters
-            Parameters instance containing all the Parameter(s)
         """
         if is_parameter(self.lnsigma):
             return self.lnsigma | self.model.parameters
@@ -348,8 +378,9 @@ class Objective(BaseObjective):
 
         Parameters
         ----------
-        pvals : np.ndarray
-            Array containing the values to be tested.
+        pvals : array-like or refnx.analysis.Parameters
+            values for the varying or entire set of parameters
+
         """
         if pvals is None:
             return
@@ -381,8 +412,8 @@ class Objective(BaseObjective):
 
         Parameters
         ----------
-        pvals : np.ndarray, optional
-            Numeric values for the Parameter's that are varying
+        pvals : array-like or refnx.analysis.Parameters
+            values for the varying or entire set of parameters
 
         Returns
         -------
@@ -391,9 +422,22 @@ class Objective(BaseObjective):
 
         Notes
         -----
-        The model attribute can also add extra terms to the log-prior if
-        needed, but it should not include any contributions from
-        `Objective.parameters` otherwise they'll be counted twice.
+        The log-prior is calculated as:
+
+        .. code-block:: python
+
+            lnprior = np.sum(param.lnprob() for param in
+                             self.varying_parameters())
+            lnprior += self.model.lnprob()
+            lnprior += self.lnprob_extra(self.model, self.data)
+
+        The major components of the log-prior probability are from the varying
+        parameters and the Model used to construct the Objective.
+        `self.model.lnprob` should not include any contributions from
+        `self.model.parameters` otherwise they'll be counted more than once.
+        The same argument applies to the user specifiable `lnprob_extra`
+        function.
+
         """
         self.setp(pvals)
 
@@ -404,6 +448,12 @@ class Objective(BaseObjective):
 
         lnprior += self.model.lnprob()
 
+        if not np.isfinite(lnprior):
+            return -np.inf
+
+        if self.lnprob_extra is not None:
+            lnprior += self.lnprob_extra(self.model, self.data)
+
         return lnprior
 
     def lnlike(self, pvals=None):
@@ -412,13 +462,28 @@ class Objective(BaseObjective):
 
         Parameters
         ----------
-        pvals : np.ndarray, optional
-            Numeric values for the Parameter's that are varying
+        pvals : array-like or refnx.analysis.Parameters
+            values for the varying or entire set of parameters
 
         Returns
         -------
         lnlike : float
             log-likelihood probability
+
+        Notes
+        -----
+        The log-likelihood is calculated as:
+
+        .. code-block:: python
+
+            lnlike = -0.5 * np.sum((y - model / s_n)**2 + np.log(s_n**2))
+
+        where
+
+        .. code-block:: python
+
+            s_n**2 = y_err**2 + exp(2 * lnsigma) * model**2
+
         """
         self.setp(pvals)
 
@@ -428,12 +493,22 @@ class Objective(BaseObjective):
 
         y, y_err, model = self._data_transform(model)
 
+        if self.lnsigma is not None:
+            var_y = (y_err * y_err +
+                     np.exp(2 * float(self.lnsigma)) * model * model)
+        else:
+            var_y = y_err ** 2
+
         # TODO do something sensible if data isn't weighted
         if self.use_weights and self.data.weighted:
             # ignoring 2 * pi constant
-            lnlike += np.log(y_err**2)
+            lnlike += np.log(var_y)
 
-        lnlike += ((y - model) / y_err)**2
+        lnlike += (y - model)**2 / var_y
+
+        # nans play havoc
+        if np.isnan(lnlike).any():
+            raise RuntimeError("Objective.lnlike encountered a NaN")
 
         return -0.5 * np.sum(lnlike)
 
@@ -443,13 +518,14 @@ class Objective(BaseObjective):
 
         Parameters
         ----------
-        pvals : np.ndarray
-            Array containing the values to be tested.
+        pvals : array-like or refnx.analysis.Parameters
+            values for the varying or entire set of parameters
 
         Returns
         -------
         nll : float
             negative log-likelihood
+
         """
         self.setp(pvals)
         return -self.lnlike()
@@ -460,8 +536,8 @@ class Objective(BaseObjective):
 
         Parameters
         ----------
-        pvals : np.ndarray, optional
-            Numeric values for the Parameter's that are varying
+        pvals : array-like or refnx.analysis.Parameters
+            values for the varying or entire set of parameters
 
         Returns
         -------
@@ -473,6 +549,7 @@ class Objective(BaseObjective):
         The overall log-probability is the sum of the log-prior and
         log-likelihood. The log-likelihood is not calculated if the log-prior
         is impossible (`lnprior == -np.inf`).
+
         """
         self.setp(pvals)
         lnprob = self.lnprior()
@@ -488,49 +565,67 @@ class Objective(BaseObjective):
     def covar(self):
         """
         Estimates the covariance matrix of the curvefitting system.
+
+        Returns
+        -------
+        covar : np.ndarray
+            Covariance matrix
+
         """
         _pvals = np.array(self.varying_parameters())
 
-        # hess = nd.Hessian(self.chisqr)(_pvals)
-
-        # if the initial attempt with unscaled values fails, than fallover
-        # to situation where we scale the parameters to 1. The initial attempt
-        # could fail, for example, if the data is scaled with a log10
-        # transform. If the nd.MaxStepGenerator makes some points negative then
-        # the transform will complain because it's trying to take the log of a
-        # negative number. By scaling the pvals to 1 in the Jacobian
-        # calculation we can get around this because the stepper won't make any
-        # of the parameter values change sign (step too small).
         used_residuals_scaler = False
 
         def residuals_scaler(vals):
             return np.squeeze(self.residuals(_pvals * vals))
 
         try:
+            # we should be able to calculate a Jacobian for a parameter whose
+            # value is zero. However, the scaling approach won't work.
+            # This will force Jacobian calculation by unscaled parameters
+            if np.any(_pvals == 0):
+                raise FloatingPointError()
+
             with np.errstate(invalid='raise'):
                 jac = approx_derivative(residuals_scaler, np.ones_like(_pvals))
             used_residuals_scaler = True
         except FloatingPointError:
             jac = approx_derivative(self.residuals, _pvals)
-
-        # if len(_pvals) == 1:
-        #     jac = jac.T
+        finally:
+            # using approx_derivative changes the state of the objective
+            # parameters have to make sure they're set at the end
+            self.setp(_pvals)
 
         # need to create this because GlobalObjective does not have
         # access to all the datapoints being fitted.
         n_datapoints = np.size(jac, 0)
 
-        # covar = J.T x J.
-        # not sure why this is preferred over Hessian
-        covar = np.linalg.inv(np.matmul(jac.T, jac))
+        # covar = J.T x J
+
+        # from scipy.optimize.minpack.py
+        # eliminates singular parameters
+        _, s, VT = np.linalg.svd(jac, full_matrices=False)
+        threshold = np.finfo(float).eps * max(jac.shape) * s[0]
+        s = s[s > threshold]
+        VT = VT[:s.size]
+        covar = np.dot(VT.T / s ** 2, VT)
 
         if used_residuals_scaler:
             # unwind the scaling.
-            covar *= _pvals ** 2
+            covar = covar * np.atleast_2d(_pvals) * np.atleast_2d(_pvals).T
+
+        pvar = np.diagonal(covar).copy()
+        psingular = np.where(pvar == 0)[0]
+
+        if len(psingular) > 0:
+            var_params = self.varying_parameters()
+            singular_params = [var_params[ps] for ps in psingular]
+
+            raise LinAlgError("The following Parameters have no effect on"
+                              " Objective.residuals, please consider fixing"
+                              " them.\n" + repr(singular_params))
 
-        self.setp(_pvals)
         scale = 1.
-
         # scale by reduced chi2 if experimental uncertainties weren't used.
         if not (self.use_weights and self.weighted):
             scale = (self.chisqr() /
@@ -557,6 +652,7 @@ class Objective(BaseObjective):
         ------
         pvec : np.ndarray
             A randomly chosen parameter vector
+
         """
         chains = np.array([np.ravel(param.chain[..., nburn::nthin]) for param
                            in self.varying_parameters()
@@ -577,7 +673,7 @@ class Objective(BaseObjective):
 
     def plot(self, pvals=None):
         """
-        Plot the data/model
+        Plot the data/model.
 
         Parameters
         ----------
@@ -586,7 +682,9 @@ class Objective(BaseObjective):
 
         Returns
         -------
-        fig, ax
+        fig, ax : :class:`matplotlib.Figure`, :class:`matplotlib.Axes`
+            `matplotlib` figure and axes objects.
+
         """
         import matplotlib.pyplot as plt
 
@@ -613,14 +711,15 @@ class GlobalObjective(Objective):
     """
     Global Objective function for simultaneous fitting with
     `refnx.analysis.CurveFitter`
+
+    Parameters
+    ----------
+    objectives : list
+        list of :class:`refnx.analysis.Objective` objects
+
     """
 
     def __init__(self, objectives):
-        """
-        Parameters
-        ----------
-        objectives : list of Objective objects
-        """
         self.objectives = objectives
         weighted = []
         use_weights = []
@@ -644,22 +743,46 @@ class GlobalObjective(Objective):
 
     @property
     def use_weights(self):
+        """
+        **bool** is data weighting being used for all the objectives?
+        """
         return self._use_weights
 
     @property
     def weighted(self):
+        """
+        **bool** do all the datasets have y_err?
+
+        """
         return self._weighted
 
     @property
     def npoints(self):
+        """
+        **int** number of data points in all the objectives.
+
+        """
         npoints = 0
         for objective in self.objectives:
             npoints += objective.npoints
         return npoints
 
     def residuals(self, pvals=None):
-        # residuals in a globalfitting context are just the individual
-        # objective.residuals concatenated.
+        """
+        Concatenated residuals for each of the
+        :meth:`refnx.analysis.Objective.residuals`.
+
+        Parameters
+        ----------
+        pvals : array-like or refnx.analysis.Parameters
+            values for the varying or entire set of parameters
+
+        Returns
+        -------
+        residuals : np.ndarray
+            Concatenated :meth:`refnx.analysis.Objective.residuals`
+
+        """
         self.setp(pvals)
 
         residuals = []
@@ -671,6 +794,10 @@ class GlobalObjective(Objective):
 
     @property
     def parameters(self):
+        """
+        :class:`refnx.analysis.Parameters` associated with all the objectives.
+
+        """
         # TODO this is probably going to be slow.
         # cache and update strategy?
         p = Parameters(name='global fitting parameters')
@@ -686,19 +813,14 @@ class GlobalObjective(Objective):
 
         Parameters
         ----------
-        pvals : np.ndarray, optional
-            Numeric values for the Parameter's that are varying
+        pvals : array-like or refnx.analysis.Parameters, optional
+            values for the varying or entire set of parameters
 
         Returns
         -------
         lnprior : float
             log-prior probability
 
-        Notes
-        -----
-        The model attribute can also add extra terms to the log-prior if
-        needed, but it should not include any contributions from
-        `Objective.parameters` otherwise they'll be counted twice.
         """
         self.setp(pvals)
 
@@ -717,13 +839,14 @@ class GlobalObjective(Objective):
 
         Parameters
         ----------
-        pvals : np.ndarray, optional
-            Numeric values for the Parameter's that are varying
+        pvals : array-like or refnx.analysis.Parameters
+            values for the varying or entire set of parameters
 
         Returns
         -------
         lnlike : float
             log-likelihood probability
+
         """
         self.setp(pvals)
         lnlike = 0.
