@@ -21,8 +21,10 @@ import refnx.util.general as general
 from refnx.util.general import resolution_double_chopper, _dict_compare
 import refnx.util.ErrorProp as EP
 from refnx.reduce.parabolic_motion import find_trajectory, y_deflection
-from refnx.reduce.event import events, process_event_stream
+from refnx.reduce.event import (events, process_event_stream,
+                                framebins_to_frames)
 from refnx.reduce.rebin import rebin, rebin_along_axis
+from refnx._lib import possibly_open_file
 
 
 # detector y pixel spacing in mm per pixel
@@ -44,15 +46,15 @@ spectrum_template = """<?xml version="1.0"?>
  spin="UNPOLARISED" dim="$n_spectra">
 <Run filename="$runnumber"/>
 <R uncertainty="dR">$r</R>
-<lambda uncertainty="dlambda" units="1/A">$l</lambda>
+<lambda uncertainty="dlambda" units="1/A">$lmda</lambda>
 <dR type="SD">$dr</dR>
-<dlambda type="_FWHM" units="1/A">$dl</dlambda>
+<dlambda type="_FWHM" units="1/A">$dlmda</dlambda>
 </REFdata>
 </REFentry>
 </REFroot>"""
 
 
-def catalogue(start, stop, path=None):
+def catalogue(start, stop, data_folder=None):
     """
     Extract interesting information from Platypus NeXUS files.
 
@@ -62,7 +64,7 @@ def catalogue(start, stop, path=None):
         start cataloguing from this run number.
     stop : int
         stop cataloguing at this run number
-    path : str, optional
+    data_folder : str, optional
         path specifying location of NeXUS files
 
     Returns
@@ -76,12 +78,12 @@ def catalogue(start, stop, path=None):
     run_number = []
     d = {key: [] for key in info}
 
-    if path is None:
-        path = '.'
+    if data_folder is None:
+        data_folder = '.'
 
     for i in range(start, stop + 1):
         try:
-            pn = PlatypusNexus(os.path.join(path, number_datafile(i)))
+            pn = PlatypusNexus(os.path.join(data_folder, number_datafile(i)))
         except OSError:
             continue
 
@@ -279,7 +281,7 @@ def basename_datafile(pth):
     return basename.split('.nx.hdf')[0]
 
 
-def number_datafile(run_number):
+def number_datafile(run_number, prefix='PLP'):
     """
     Given a run number figure out what the file name is.
     Given a file name, return the filename with the .nx.hdf extension
@@ -288,6 +290,9 @@ def number_datafile(run_number):
     ----------
     run_number : int or str
 
+    prefix : str, optional
+        The instrument prefix. Only used if `run_number` is an int
+
     Returns
     -------
     file_name : str
@@ -295,14 +300,16 @@ def number_datafile(run_number):
     Examples
     --------
     >>> number_datafile(708)
-    'PLP0000708.nx.hdf
+    'PLP0000708.nx.hdf'
+    >>> number_datafile(708, prefix='QKK')
+    'QKK0000708.nx.hdf'
     >>> number_datafile('PLP0000708.nx.hdf')
     'PLP0000708.nx.hdf'
     """
     try:
         num = abs(int(run_number))
         # you got given a run number
-        return 'PLP{0:07d}.nx.hdf'.format(num)
+        return '{0}{1:07d}.nx.hdf'.format(prefix, num)
     except ValueError:
         # you may have been given full filename
         if run_number.endswith('.nx.hdf'):
@@ -324,10 +331,12 @@ def datafile_number(fname):
     -------
     run_number : int
         The run number
+
     Examples
     --------
     >>> datafile_number('PLP0000708.nx.hdf')
     708
+
     """
 
     regex = re.compile(".*PLP([0-9]{7}).nx.hdf")
@@ -340,22 +349,8 @@ def datafile_number(fname):
     return None
 
 
-class PlatypusNexus(object):
-    """
-    Processes Platypus NeXus files to produce an intensity vs wavelength
-    spectrum
-
-    Parameters
-    ----------
-    h5data : HDF5 NeXus file or str
-        An HDF5 NeXus file for Platypus, or a `str` containing the path
-        to one
-    """
-
-    def __init__(self, h5data):
-        """
-        Initialises the PlatypusNexus object.
-        """
+class ReflectNexus(object):
+    def __init__(self, h5data, prefix):
         with _possibly_open_hdf_file(h5data, 'r') as f:
             self.cat = Catalogue(f)
 
@@ -366,6 +361,8 @@ class PlatypusNexus(object):
         # call process again, thereby saving time.
         self._arguments = {}
 
+        self.prefix = prefix
+
     def __getattr__(self, item):
         if item in self.__dict__:
             return self.__dict__[item]
@@ -374,7 +371,7 @@ class PlatypusNexus(object):
         else:
             raise AttributeError
 
-    def __short_circuit_process(self, _arguments):
+    def _short_circuit_process(self, _arguments):
         """
         Returns the truth that two sets of arguments from successive calls to
         the `process` method are the same.
@@ -393,13 +390,122 @@ class PlatypusNexus(object):
 
         return _dict_compare(_arguments, self._arguments)
 
+    def write_spectrum_dat(self, f, scanpoint=0):
+        """
+        This method writes a dat representation of the corrected spectrum to
+        file.
+
+        Parameters
+        ----------
+        f : file-like or str
+            The file to write the spectrum to, or a str that specifies the file
+            name
+        scanpoint : int
+            Which scanpoint to write
+
+        Returns
+        -------
+        processed : bool
+            If the file hasn't been processed then the `processed is False` and
+            vice versa
+        """
+        if self.processed_spectrum is None:
+            return False
+
+        m_lambda = self.processed_spectrum['m_lambda'][scanpoint]
+        m_spec = self.processed_spectrum['m_spec'][scanpoint]
+        m_spec_sd = self.processed_spectrum['m_spec_sd'][scanpoint]
+        m_lambda_fwhm = self.processed_spectrum['m_lambda_fwhm'][scanpoint]
+
+        stacked_data = np.c_[m_lambda, m_spec, m_spec_sd, m_lambda_fwhm]
+        np.savetxt(f, stacked_data, delimiter='\t')
+
+        return True
+
+    def write_spectrum_xml(self, f, scanpoint=0):
+        """
+        This method writes an XML representation of the corrected spectrum to
+        file.
+
+        Parameters
+        ----------
+        f : file-like or str
+            The file to write the spectrum to, or a str that specifies the file
+            name
+        scanpoint : int
+            Which scanpoint to write
+        """
+        if self.processed_spectrum is None:
+            return
+
+        s = string.Template(spectrum_template)
+        d = dict()
+        d['title'] = self.cat.sample_name
+        d['time'] = strftime("%a, %d %b %Y %H:%M:%S +0000", gmtime())
+
+        m_lambda = self.processed_spectrum['m_lambda']
+        m_spec = self.processed_spectrum['m_spec']
+        m_spec_sd = self.processed_spectrum['m_spec_sd']
+        m_lambda_fwhm = self.processed_spectrum['m_lambda_fwhm']
+
+        # sort the data
+        sorted = np.argsort(self.m_lambda[0])
+
+        r = m_spec[:, sorted]
+        lmda = m_lambda[:, sorted]
+        dlmda = m_lambda_fwhm[:, sorted]
+        dr = m_spec_sd[:, sorted]
+        d['n_spectra'] = self.processed_spectrum['n_spectra']
+        d['runnumber'] = 'PLP{:07d}'.format(self.cat.datafile_number)
+
+        d['r'] = repr(r[scanpoint].tolist()).strip(',[]')
+        d['dr'] = repr(dr[scanpoint].tolist()).strip(',[]')
+        d['lmda'] = repr(lmda[scanpoint].tolist()).strip(',[]')
+        d['dlmda'] = repr(dlmda[scanpoint].tolist()).strip(',[]')
+        thefile = s.safe_substitute(d)
+
+        with possibly_open_file(f, 'wb') as g:
+            if 'b' in g.mode:
+                thefile = thefile.encode('utf-8')
+
+            g.write(thefile)
+            g.truncate()
+
+        return True
+
+    @property
+    def spectrum(self):
+        return (self.processed_spectrum['m_lambda'],
+                self.processed_spectrum['m_spec'],
+                self.processed_spectrum['m_spec_sd'],
+                self.processed_spectrum['m_lambda_fwhm'])
+
+
+class PlatypusNexus(ReflectNexus):
+    """
+    Processes Platypus NeXus files to produce an intensity vs wavelength
+    spectrum
+
+    Parameters
+    ----------
+    h5data : HDF5 NeXus file or str
+        An HDF5 NeXus file for Platypus, or a `str` containing the path
+        to one
+    """
+
+    def __init__(self, h5data):
+        """
+        Initialises the PlatypusNexus object.
+        """
+        super(PlatypusNexus, self).__init__(h5data, 'PLP')
+
     def process(self, h5norm=None, lo_wavelength=2.5, hi_wavelength=19.,
                 background=True, direct=False, omega=None, twotheta=None,
                 rebin_percent=1., wavelength_bins=None, normalise=True,
                 integrate=-1, eventmode=None, event_folder=None,
                 peak_pos=None, peak_pos_tol=0.01,
                 background_mask=None, normalise_bins=True,
-                manual_beam_find=None, **kwds):
+                manual_beam_find=None, event_filter=None, **kwds):
         r"""
         Processes the ProcessNexus object to produce a time of flight spectrum.
         The processed spectrum is stored in the `processed_spectrum` attribute.
@@ -434,13 +540,14 @@ class PlatypusNexus(object):
         normalise : bool
             Normalise by the monitor counts.
         integrate : int
+            Specifies which scanpoints to use.
 
-            - integrate == -1
-              the spectrum is integrated over all the scanpoints.
-            - integrate >= 0
-              the individual spectra are calculated individually.
-              If `eventmode is not None` then integrate specifies which
-              scanpoint to examine.
+             - integrate == -1
+               the spectrum is integrated over all the scanpoints.
+             - integrate >= 0
+               the individual spectra are calculated individually.
+               If `eventmode is not None`, or `event_filter is not None` then
+               integrate specifies which scanpoint to examine.
 
         eventmode : None or array_like
             If eventmode is `None` then the integrated detector image is used.
@@ -452,11 +559,12 @@ class PlatypusNexus(object):
             If eventmode has zero length (e.g. []), then a single time interval
             for the entire acquisition is used, [0, acquisition_time].  This
             would source the image from the eventmode file, rather than the
-            NeXUS file. Note: the two approaches will probably not give
+            NeXUS file. The two approaches will probably not give
             identical results, because the eventmode method adjusts the total
             acquisition time and beam monitor counts to the frame number of the
             last event detected (which may be quite different if the count rate
-            is very low).
+            is very low). This parameter is disregarded if `event_filter` is
+            provided.
         event_folder : None or str
             Specifies the path for the eventmode data. If
             `event_folder is None` then the eventmode data is assumed to reside
@@ -466,10 +574,14 @@ class PlatypusNexus(object):
             Options for finding specular peak position and peak standard
             deviation.
 
-                -1             - use `manual_beam_find`.
-                None           - use the automatic beam finder, falling back to
-                                 `manual_beam_find` if it's provided.
-                (float, float) - specify the peak and peak standard deviation.
+            - \-1
+               use `manual_beam_find`.
+            - None
+               use the automatic beam finder, falling back to
+               `manual_beam_find` if it's provided.
+            - (float, float)
+               specify the peak and peak standard deviation.
+
         peak_pos_tol : float
             Convergence tolerance for the beam position and width to be
             accepted from successive beam-finder calculations; see the
@@ -498,6 +610,27 @@ class PlatypusNexus(object):
             `background_pixels` is a list of length `n`. Each of the entries
             should contain arrays of pixel numbers that specify the background
             region for each of the detector images.
+        event_filter : callable, optional
+            A function, that processes the event stream, returning a `detector`
+            array, and a `frame_count` array. `detector` has shape
+            `(N, T, Y, X)`, where `N` is the number of detector images, `T` is
+            the number of time bins (`len(t_bins)`), etc. `frame_count` has
+            shape `(N,)` and contains the number of frames for each of the
+            detector images. The frame_count is used to determine what fraction
+            of the overall monitor counts should be ascribed to each detector
+            image (by dividing by the total number of frames). The function has
+            signature:
+
+            detector, frame_count = event_filter(loaded_events,
+                                                 t_bins=None,
+                                                 y_bins=None,
+                                                 x_bins=None)
+
+            `loaded_events` is a 4-tuple of numpy arrays:
+            `(f_events, t_events, y_events, x_events)`, where `f_events`
+            contains the frame number for each neutron, landing at position
+            `x_events, y_events` on the detector, with time-of-flight
+            `t_events`.
 
         Notes
         -----
@@ -530,6 +663,7 @@ class PlatypusNexus(object):
         m_lambda, m_spec, m_spec_sd: np.ndarray
             Arrays containing the wavelength, specular intensity as a function
             of wavelength, standard deviation of specular intensity
+
         """
 
         # it can be advantageous to save processing time if the arguments
@@ -539,7 +673,7 @@ class PlatypusNexus(object):
         _arguments.pop('self', None)
         # if you've already processed, then you may not need to process again
         if (self.processed_spectrum and
-                self.__short_circuit_process(_arguments)):
+                self._short_circuit_process(_arguments)):
             return (self.processed_spectrum['m_lambda'],
                     self.processed_spectrum['m_spec'],
                     self.processed_spectrum['m_spec_sd'])
@@ -558,20 +692,22 @@ class PlatypusNexus(object):
 
         # This section controls how multiple detector images are handled.
         # We want event streaming.
-        if eventmode is not None:
+        if eventmode is not None or event_filter is not None:
             scanpoint = integrate
             if integrate == -1:
                 scanpoint = 0
 
             output = self.process_event_stream(scanpoint=scanpoint,
                                                frame_bins=eventmode,
-                                               event_folder=event_folder)
-            frame_bins, detector, bm1_counts = output
+                                               event_folder=event_folder,
+                                               event_filter=event_filter)
+            detector, frame_count, bm1_counts = output
 
             start_time = np.zeros(np.size(detector, 0))
             if cat.start_time is not None:
                 start_time += cat.start_time[scanpoint]
-                start_time += frame_bins[:-1]
+                start_time[1:] += (np.cumsum(frame_count)[:-1] /
+                                   cat.frequency[scanpoint])
         else:
             # we don't want detector streaming
             detector = cat.detector
@@ -681,7 +817,7 @@ class PlatypusNexus(object):
 
             detpositions[idx] = cat.dy[scanpoint]
 
-            if eventmode is not None:
+            if eventmode is not None or event_filter is not None:
                 m_spec_tof_hist[:] = TOF - toffset
                 flight_distance[:] = flight_distance[0]
                 detpositions[:] = detpositions[0]
@@ -747,7 +883,7 @@ class PlatypusNexus(object):
 
         if np.size(beam_centre) != n_spectra:
             raise RuntimeError('The number of beam centres should be equal'
-                               'to the number of detector images.')
+                               ' to the number of detector images.')
 
         '''
         Rebinning in lambda for all detector
@@ -1069,7 +1205,8 @@ class PlatypusNexus(object):
         return chod, d_cx
 
     def process_event_stream(self, t_bins=None, x_bins=None, y_bins=None,
-                             frame_bins=None, scanpoint=0, event_folder=None):
+                             frame_bins=None, scanpoint=0, event_folder=None,
+                             event_filter=None):
         """
         Processes the event mode dataset for the NeXUS file. Assumes that
         there is a event mode directory in the same directory as the NeXUS
@@ -1077,13 +1214,6 @@ class PlatypusNexus(object):
 
         Parameters
         ----------
-        frame_bins : array_like, optional
-            specifies the frame bins required in the image. If
-            framebins = [5, 10, 120] you will get 2 images.  The first starts
-            at 5s and finishes at 10s. The second starts at 10s and finishes
-            at 120s. If frame_bins has zero length, e.g. [], then a single
-            interval consisting of the entire acquisition time is used:
-            [0, acquisition_time].
         t_bins : array_like, optional
             specifies the time bins required in the image
         x_bins : array_like, optional
@@ -1097,10 +1227,38 @@ class PlatypusNexus(object):
             `event_folder is None` then the eventmode data is assumed to reside
             in the same directory as the NeXUS file. If event_folder is a
             string, then the string specifies the path to the eventmode data.
+        frame_bins : array_like, optional
+            specifies the frame bins required in the image. If
+            frame_bins = [5, 10, 120] you will get 2 images.  The first starts
+            at 5s and finishes at 10s. The second starts at 10s and finishes
+            at 120s. If frame_bins has zero length, e.g. [], then a single
+            interval consisting of the entire acquisition time is used:
+            [0, acquisition_time]. If `event_filter` is provided then this
+            parameter is ignored.
+        event_filter : callable, optional
+            A function, that processes the event stream, returning a `detector`
+            array, and a `frame_count` array. `detector` has shape
+            `(N, T, Y, X)`, where `N` is the number of detector images, `T` is
+            the number of time bins (`len(t_bins)`), etc. `frame_count` has
+            shape `(N,)` and contains the number of frames for each of the
+            detector images. The frame_count is used to determine what fraction
+            of the overall monitor counts should be ascribed to each detector
+            image. The function has signature:
+
+            detector, frame_count = event_filter(loaded_events,
+                                                 t_bins=None,
+                                                 y_bins=None,
+                                                 x_bins=None)
+
+            `loaded_events` is a 4-tuple of numpy arrays:
+            `(f_events, t_events, y_events, x_events)`, where `f_events`
+            contains the frame number for each neutron, landing at position
+            `x_events, y_events` on the detector, with time-of-flight
+            `t_events`.
 
         Returns
         -------
-        frame_bins, detector, bm1_counts
+        detector, frame_count, bm1_counts : np.ndarray, np.ndarray, np.ndarray
 
         Create a new detector image based on the t_bins, x_bins, y_bins and
         frame_bins you supply to the method (these should all be lists/numpy
@@ -1109,10 +1267,12 @@ class PlatypusNexus(object):
         would essentially return the same detector image as the nexus file.
         However, you can specify the frame_bins list to generate detector
         images based on subdivided periods of the total acquisition.
-        For example if framebins = [5, 10, 120] you will get 2 images.  The
+        For example if frame_bins = [5, 10, 120] you will get 2 images.  The
         first starts at 5s and finishes at 10s. The second starts at 10s
         and finishes at 120s. The frame_bins are clipped to the total
         acquisition time if necessary.
+        `frame_count` is how many frames went into making each detector image.
+
         """
         cat = self.cat
 
@@ -1146,126 +1306,32 @@ class PlatypusNexus(object):
             loaded_events, end_of_last_event = events(f,
                                                       max_frames=last_frame)
 
-        output = process_event_stream(loaded_events,
-                                      np.asfarray(frame_bins) *
-                                      frequency,
-                                      t_bins,
-                                      y_bins,
-                                      x_bins)
+        # convert frame_bins to list of filter frames
+        frames = framebins_to_frames(np.asfarray(frame_bins) *
+                                     frequency)
 
-        detector, new_frame_bins = output
+        if event_filter is not None:
+            output = event_filter(loaded_events, t_bins, y_bins, x_bins)
+        else:
+            output = process_event_stream(loaded_events,
+                                          frames,
+                                          t_bins,
+                                          y_bins,
+                                          x_bins)
 
-        new_frame_bins /= frequency
+        detector, frame_count = output
 
-        bm1_counts = new_frame_bins[1:] - new_frame_bins[:-1]
-        bm1_counts *= (bm1_counts_for_scanpoint / total_acquisition_time)
+        bm1_counts = (frame_count * bm1_counts_for_scanpoint /
+                      total_acquisition_time /
+                      frequency)
 
-        return new_frame_bins, detector, bm1_counts
-
-    def write_spectrum_dat(self, f, scanpoint=0):
-        """
-        This method writes a dat representation of the corrected spectrum to
-        file.
-
-        Parameters
-        ----------
-        f : file-like or str
-            The file to write the spectrum to, or a str that specifies the file
-            name
-        scanpoint : int
-            Which scanpoint to write
-
-        Returns
-        -------
-        processed : bool
-            If the file hasn't been processed then the `processed is False` and
-            vice versa
-        """
-        if self.processed_spectrum is None:
-            return False
-
-        m_lambda = self.processed_spectrum['m_lambda'][scanpoint]
-        m_spec = self.processed_spectrum['m_spec'][scanpoint]
-        m_spec_sd = self.processed_spectrum['m_spec_sd'][scanpoint]
-        m_lambda_fwhm = self.processed_spectrum['m_lambda_fwhm'][scanpoint]
-
-        stacked_data = np.c_[m_lambda, m_spec, m_spec_sd, m_lambda_fwhm]
-        np.savetxt(f, stacked_data, delimiter='\t')
-
-        return True
-
-    def write_spectrum_xml(self, f, scanpoint=0):
-        """
-        This method writes an XML representation of the corrected spectrum to
-        file.
-
-        Parameters
-        ----------
-        f : file-like or str
-            The file to write the spectrum to, or a str that specifies the file
-            name
-        scanpoint : int
-            Which scanpoint to write
-        """
-        if self.processed_spectrum is None:
-            return
-
-        s = string.Template(spectrum_template)
-        d = dict()
-        d['title'] = self.cat.sample_name
-        d['time'] = strftime("%a, %d %b %Y %H:%M:%S +0000", gmtime())
-
-        m_lambda = self.processed_spectrum['m_lambda']
-        m_spec = self.processed_spectrum['m_spec']
-        m_spec_sd = self.processed_spectrum['m_spec_sd']
-        m_lambda_fwhm = self.processed_spectrum['m_lambda_fwhm']
-
-        # sort the data
-        sorted = np.argsort(self.m_lambda[0])
-
-        r = m_spec[:, sorted]
-        l = m_lambda[:, sorted]
-        dl = m_lambda_fwhm[:, sorted]
-        dr = m_spec_sd[:, sorted]
-        d['n_spectra'] = self.processed_spectrum['n_spectra']
-        d['runnumber'] = 'PLP{:07d}'.format(self.cat.datafile_number)
-
-        d['r'] = repr(r[scanpoint].tolist()).strip(',[]')
-        d['dr'] = repr(dr[scanpoint].tolist()).strip(',[]')
-        d['l'] = repr(l[scanpoint].tolist()).strip(',[]')
-        d['dl'] = repr(dl[scanpoint].tolist()).strip(',[]')
-        thefile = s.safe_substitute(d)
-
-        g = f
-        auto_fh = None
-
-        if not hasattr(f, 'write'):
-            auto_fh = open(f, 'wb')
-            g = auto_fh
-
-        if 'b' in g.mode:
-            thefile = thefile.encode('utf-8')
-
-        g.write(thefile)
-        g.truncate()
-
-        if auto_fh is not None:
-            auto_fh.close()
-
-        return True
-
-    @property
-    def spectrum(self):
-        return (self.processed_spectrum['m_lambda'],
-                self.processed_spectrum['m_spec'],
-                self.processed_spectrum['m_spec_sd'],
-                self.processed_spectrum['m_lambda_fwhm'])
+        return detector, frame_count, bm1_counts
 
 
 def create_detector_norm(h5norm, x_min, x_max):
     """
-    Produces a detector normalisation array for Platypus.
-    Here we average over N, T and X to provide  a relative efficiency for each
+    Produces a detector normalisation array for a neutron detector.
+    Here we average over N, T and X to provide a relative efficiency for each
     y wire.
 
     Parameters
@@ -1418,7 +1484,7 @@ def background_subtract_line(profile, profile_sd, background_mask):
     return EP.EPsub(profile, profile_sd, bkgd, bkgd_sd)
 
 
-def find_specular_ridge(detector, detector_sd, starting_offset=50,
+def find_specular_ridge(detector, detector_sd, search_increment=50,
                         tol=0.01, manual_beam_find=None):
     """
     Find the specular ridges in a detector(n, t, y) plot.
@@ -1429,6 +1495,8 @@ def find_specular_ridge(detector, detector_sd, starting_offset=50,
         detector array
     detector_sd : array_like
         standard deviations of detector array
+    search_increment : int
+        specifies the search increment for the location process.
     tol : float
         specifies threshold of fractional change for beam centre to be found
     manual_beam_find : callable, optional
@@ -1454,6 +1522,16 @@ def find_specular_ridge(detector, detector_sd, starting_offset=50,
         background region, array specifying points to be used for background
         subtraction
         `np.size(centre) == n`.
+
+    Notes
+    -----
+    The search for the beam centre proceeds by taking the last
+    `search_increment` time bins, summing over the time axis and finding
+    the beam centre and width along the y-axis. It then repeats the process
+    with the last `2 * search_increment` time bins. This process is repeated
+    until the relative change in beam centre and width is lower than `tol`.
+    This process is designed to locate the specular ridge, even in the
+    presence of incoherent scattering.
     """
     beam_centre = np.zeros(np.size(detector, 0))
     beam_sd = np.zeros_like(beam_centre)
@@ -1467,9 +1545,9 @@ def find_specular_ridge(detector, detector_sd, starting_offset=50,
 
     search_increment = 50
 
-    starting_offset = abs(starting_offset)
+    search_increment = abs(search_increment)
 
-    n_increments = ((np.size(detector, 1) - starting_offset) //
+    n_increments = ((np.size(detector, 1) - search_increment) //
                     search_increment)
 
     # we want to integrate over the following pixel region
@@ -1479,7 +1557,7 @@ def find_specular_ridge(detector, detector_sd, starting_offset=50,
         converged = False
 
         for i in range(n_increments):
-            how_many = -starting_offset - search_increment * i
+            how_many = -search_increment - search_increment * i
 
             det_subset = detector[j, -1: how_many: -1]
             det_sd_subset = detector_sd[j, -1: how_many: -1]
